@@ -4,7 +4,7 @@ package Terminal::Identify;
 # terminal emulator which is in use by the logged-in user.
 #
 # The Linux command ps can be used to do this: 
-# ps -o 'cmd=' -p $(ps -o 'ppid=' -p $$
+# ps -o 'cmd=' -p $(ps -o 'ppid=' -p $$)
 # ------------------------------------------------------------------------------
 
 # Load the basic Perl pragmas.
@@ -29,7 +29,7 @@ our @EXPORT = qw(
 our @ISA = qw(Exporter);
 
 # Set the package version. 
-our $VERSION = '0.09';
+our $VERSION = '0.10';
 
 # Load the Perl modules.
 use POSIX qw(ttyname);
@@ -182,7 +182,7 @@ sub login_users {
     my @user_array = split ' ', $user;
     # Return the array with the logged-in users.
     return @user_array;
-}
+};
 
 #------------------------------------------------------------------------------# 
 # Subroutine login_shells                                                      #
@@ -247,150 +247,6 @@ sub get_ppid {
     return $ppid;
 };
 
-#------------------------------------------------------------------------------# 
-# Subroutine terminal_command_line                                             #
-#                                                                              #
-# Description:                                                                 #
-# Use the Linux command ps to get the command line of the process which is     #
-# related to the terminal in use.                                              #
-#                                                                              #
-# @argument  $_[0]      PPID                   (scalar)                        #
-# @return    $termproc  Terminal command line  (scalar)                        #
-#------------------------------------------------------------------------------# 
-sub terminal_command_line {
-    # Assign the subroutine argument to the local variable $ppid.
-    my $ppid = $_[0];
-    # Get the command column from the ps output.
-    my $termproc = `ps --no-headers -o cmd:1 -p $ppid 2>/dev/null`;
-    # Trim the command line output string.  
-    $termproc = trim($termproc);
-    # Return the process related to the terminal in use.
-    return $termproc
-};
-
-#------------------------------------------------------------------------------# 
-# Subroutine user_shell                                                        #
-#                                                                              #
-# Description:                                                                 #
-# Get the user login shells.                                                   #
-#                                                                              #
-# @arguments: $_[0]     Content of passwd       (scalar)                       # 
-#             @{$_[1]}  User array              (array)                        #
-#             @{$_[2]}  Shell array             (array)                        #
-# @return:    @match    Array with the matches  (array)                        #
-#------------------------------------------------------------------------------# 
-sub user_shell {
-    # Assign the subroutine arguments to the local variables.
-    my $passwd_content = $_[0];
-    my @user_array = @{$_[1]};
-    my @shell_array = @{$_[2]};
-    # Declare the array @match.
-    my @match = ();
-    # Loop over the array with the lines of $passwd_content.
-    foreach (split '\n', $passwd_content) {
-        # Get user and shell from each line of content.
-        my ($user) = $_ =~ /^(.*?):.*/;
-        my ($shell) = $_ =~ /^.*\/(.*)$/;
-        # Check user and shall against the given arrays.
-        if (grep(/^$user$/, @user_array) && grep(/^$shell$/, @shell_array)) {
-            # Add line to array @match.
-            push(@match, $_);
-        };
-    };
-    # Return the array @match.
-    return @match;
-};
-
-#------------------------------------------------------------------------------# 
-# Subroutine user_term                                                         #
-#                                                                              #
-# Description:                                                                 #
-# Get the pseudo terminal slave, which is related to the executed Perl script. #
-#                                                                              #
-# @argument: @{$_[0]}  Array with the matches    (array)                       # 
-# @return:   @result   Array with the processes  (array)                       #
-#------------------------------------------------------------------------------# 
-sub user_term {
-    # Assign the subroutine argument to the local variable.
-    my @match = @{$_[0]};
-    # Declare the array @result.
-    my @result;
-    # Get the terminal path.
-    # my $term_path = TermPath();
-    my $fileno = fileno(STDIN);
-    my $term_path = ttyname($fileno);
-    # Check if $term_path is defined, otherwise set $term_path to "".
-    $term_path = (defined $term_path ? $term_path : "");
-    # Extract the pseudo terminal slave. 
-    my ($term) = $term_path =~ /^.*\/dev\/(pts\/\d+)$/;
-    # Return if $term is empty.
-    if (!defined $term) {
-        # Return empty array. 
-        return ();
-    };
-    # Add square brackets to searchstring.
-    $term = search_brackets($term);
-    # Loop over the matches (lines of passwd).
-    foreach (@match) {
-        # Get user and shell from given line. 
-        my ($user) = $_ =~ /^(.*?):.*/;
-        my ($shell) = $_ =~ /^.*\/(.*)$/;
-        # Add square brackets to searchstrings.
-        $user = search_brackets($user);
-        $shell = search_brackets($shell);
-        # Search for process which is matching user, shell and terminal.
-        my $process = `ps aux | grep $shell | grep $user | grep $term`;
-        # Check if result is not empty.
-        if ($process ne '') {
-            # Add process to array @result.
-            push(@result, $process);
-        };
-    };
-    # Return the array @result.
-    return @result;
-};
-
-#------------------------------------------------------------------------------# 
-# Subroutine process_command_line                                              #
-#                                                                              #
-# Description:                                                                 #
-# Identify the process command line of the terminal in use.                    #
-#                                                                              #
-# @argument: None                                                              # 
-# @return:   $terminal  Process command line  (scalar)                         #
-#------------------------------------------------------------------------------# 
-sub process_command_line {
-    # Declare the return variable $terminal.
-    my $terminal;
-    # Set the filename.
-    my $filename = "/etc/passwd";
-    # Get the logged in users.
-    my @user_arr = login_users();
-    # Get the available login shells.
-    my @shell_arr = login_shells();
-    # Read the file /etc/passwd in variable $content.
-    my $content = read_file($filename);
-    # Create the array with user and shell.
-    my @match = user_shell($content, \@user_arr, \@shell_arr);
-    # Create the array with user and term.
-    @match = user_term(\@match);
-    if (@match > 0) {
-        # Split up the process by lines and white spaces.
-        my @columns = split /\s+/, $match[0];
-        # Extract the PID from the array.
-        my $pid = $columns[1];
-        # Get the PPID from the command ps.
-        my $ppid = get_ppid($pid);
-        # Get the terminal in use from the command ps.
-        $terminal = terminal_command_line($ppid);
-    } else {
-        # Set variable $terminal to NONE.
-        $terminal = "NONE";
-    };
-    # Return the terminal process command or NONE.
-    return $terminal
-};
-
 #------------------------------------------------------------------------------# 
 # Subroutine remove_arguments                                                  #
 #                                                                              #
@@ -418,7 +274,163 @@ sub remove_arguments {
     };
     # Return the new parts array.
     return @new_parts;
-}
+};
+
+#------------------------------------------------------------------------------# 
+# Subroutine term_user_shell                                                   #
+#                                                                              #
+# Description:                                                                 #
+# Create a multi dimensional array with term, user and shell.                  #
+#                                                                              #
+# @arguments: $_[0] => $passwd_content  Content of passwd      (scalar)        # 
+# @return:    @result_array             Array with the result  (array)         #
+#------------------------------------------------------------------------------# 
+sub term_user_shell {
+    # Assign the subroutine arguments to the local variables.
+    my $passwd_content = $_[0];
+    my @user_array = @{$_[1]};
+    my @shell_array = @{$_[2]};
+    # Get the username related to the terminal.
+    my $username = (getlogin() || (getpwuid($<))[0] ||
+                    $ENV{LOGNAME} || $ENV{USER});
+    # Declare the return array.
+    my @result_array = ();
+    # Get the terminal path.
+    # my $term_path = TermPath();
+    my $fileno = fileno(STDIN);
+    my $term_path = ttyname($fileno);
+    # Check if $term_path is defined, otherwise set $term_path to "".
+    $term_path = (defined $term_path ? $term_path : "");
+    # Extract the pseudo terminal slave. 
+    my ($term) = $term_path =~ /^.*\/dev\/(pts\/\d+)$/;
+    # If $term is empty return empty array.
+    if (!defined $term) {
+        # Return empty array. 
+        return ();
+    };
+    # Loop over the array with the lines of $passwd_content.
+    foreach (split '\n', $passwd_content) {
+        # Get user and shell from each line of the content.
+        my ($user) = $_ =~ /^(.*?):.*/;
+        my ($shell) = $_ =~ /^.*\/(.*)$/;
+        if ($shell ne "nologin" && $shell ne "sync" && $shell ne "false") {
+            # Check user and shell against the given arrays.
+            if (grep(/^$user$/, @user_array) &&
+                grep(/^$shell$/, @shell_array)) {
+                # Assemble a new list.  
+                my @tmp = ($term, $user, $shell); 
+                # Add data to array.
+                push(@result_array, \@tmp);
+            } elsif ($user eq $username) {
+                # Assemble a new list.  
+                my @tmp = ($term, $user, $shell); 
+                # Add data to array.
+                push(@result_array, \@tmp);
+            }; 
+        };
+    };
+    # Return the array.
+    return @result_array;
+};
+
+#------------------------------------------------------------------------------# 
+# Subroutine terminal_process                                                  #
+#                                                                              #
+# Description:                                                                 # 
+# Get the process related to term, user and shell.                             #
+#                                                                              #
+# @argument: @{$_[0]}  Array with term, user and shell  (array)                # 
+# @return:   @result   Array with matching processes    (array)                #
+#------------------------------------------------------------------------------# 
+sub terminal_process {
+    # Assign the subroutine argument to the local variable.
+    my @input_array = @{$_[0]};
+    # Declare the array @result.
+    my @match;
+    # Loop over the matches (lines of passwd).
+    foreach my $item (@input_array) {
+        # Get term, user and shell from array.
+        my $term = $item->[0];
+        my $user = $item->[1];
+        my $shell = $item->[2];
+        # Add square brackets to searchstrings.
+        $term = search_brackets($term);
+        $user = search_brackets($user);
+        $shell = search_brackets($shell);
+        # Search for process which is matching user, shell and terminal.
+        my $process = `ps aux | grep $shell | grep $user | grep $term`;
+        # Check if result is not empty.
+        if ($process ne '') {
+            # Add process to array @result.
+            push(@match, $process);
+        };
+    };
+    # Return the array.
+    return @match;
+};
+
+#------------------------------------------------------------------------------# 
+# Subroutine terminal_command_line                                             #
+#                                                                              #
+# Description:                                                                 #
+# Use the Linux command ps to get the command line of the process which is     #
+# related to the terminal in use.                                              #
+#                                                                              #
+# @argument  $_[0]      PPID                   (scalar)                        #
+# @return    $termproc  Terminal command line  (scalar)                        #
+#------------------------------------------------------------------------------# 
+sub terminal_command_line {
+    # Assign the subroutine argument to the local variable $ppid.
+    my $ppid = $_[0];
+    # Get the command column from the ps output.
+    my $termproc = `ps --no-headers -o cmd:1 -p $ppid 2>/dev/null`;
+    # Trim the command line output string.  
+    $termproc = trim($termproc);
+    # Return the process related to the terminal in use.
+    return $termproc
+};
+
+#------------------------------------------------------------------------------# 
+# Subroutine terminal_identifier                                               #
+#                                                                              #
+# Description:                                                                 #
+# Identify the process command line of the terminal in use.                    #
+#                                                                              #
+# @argument: None                                                              # 
+# @return:   $terminal  Process command line  (scalar)                         #
+#------------------------------------------------------------------------------# 
+sub terminal_identifier {
+    # Declare the return variable $terminal.
+    my $terminal;
+    # Set the filename.
+    my $filename = "/etc/passwd";
+    # Get the logged in users.
+    my @user_arr = login_users();
+    # Get the available login shells.
+    my @shell_arr = login_shells();
+    # Read the file /etc/passwd in and store it in variable $content.
+    my $content = read_file($filename);
+    # Create the array with user and shell.
+    my @result = term_user_shell($content, \@user_arr, \@shell_arr);
+    # Create the array with user and term.
+    my @match = terminal_process(\@result);
+    # Check if array is empty.
+    if (@match > 0) {
+        # Split up the process by lines and white spaces.
+        my @columns = split /\s+/, $match[0];
+        # Extract the PID from the array.
+        my $pid = $columns[1];
+        # Get the PPID from the command ps.
+        my $ppid = get_ppid($pid);
+        # Get the terminal in use from the command ps.
+        $terminal = terminal_command_line($ppid);
+    } else {
+        # Set variable $terminal to NONE.
+        $terminal = "NONE";
+    };
+    # Return the terminal process command or NONE.
+    return $terminal
+};
 
 #------------------------------------------------------------------------------# 
 # Subroutine whichterminalami                                                  # 
@@ -438,7 +450,7 @@ sub whichterminalami {
     my $terminal;
     my $terminal_ftn;
     # Identify the terminal process command.
-    my $terminal_process = process_command_line();
+    my $terminal_process = terminal_identifier();
     # If terminal process is set to NONE it is the system console.
     if ($terminal_process eq "NONE") {
         # Return the system console.
